@@ -21,6 +21,7 @@
 import ConfigParser
 from   gettext             import gettext
 import logging
+import re
 import os
 import optparse
 import shutil
@@ -52,7 +53,7 @@ class VM(object):
     
 
     """
-    def __init__(self):
+    def __init__(self, conf=None):
         self.hypervisor = None #: hypervisor object, representing the hypervisor the vm is destined for
         self.distro = None
 
@@ -72,11 +73,20 @@ class VM(object):
         #: directory where we build up the guest filesystem
         self.tmproot = None
 
+        self.fsmounted = False
+
         self.optparser = _MyOptParser(epilog="ubuntu-vm-builder is Copyright (C) 2007-2008 Canonical Ltd. and written by Soren Hansen <soren@canonical.com>.", usage='%prog hypervisor distro [options]')
         self.optparser.arg_help = (('hypervisor', self.hypervisor_help), ('distro', self.distro_help))
 
         self.confparser = ConfigParser.SafeConfigParser()
-        self.confparser.read(['/etc/vmbuilder.cfg', os.path.expanduser('~/.vmbuilder.cfg')])
+
+        if conf:
+            if not(os.path.isfile(conf)):
+                raise VMBuilderUserError('The path to the configuration file is not valid: %s.' % conf)
+        else:
+            conf = ''
+
+        self.confparser.read(['/etc/vmbuilder.cfg', os.path.expanduser('~/.vmbuilder.cfg'), conf])
 
         self._register_base_settings()
 
@@ -116,21 +126,24 @@ class VM(object):
         return optparse.OptionGroup(self.optparser, *args, **kwargs)
 
     def _register_base_settings(self):
-        self.register_setting('-c', dest='altconfig', default='~/.ubuntu-vm-builder', help='Specify a optional configuration file [default: %default]')
-        self.register_setting('-d', '--dest', help='Specify the destination directory. [default: <hypervisor>-<distro>]')
+        self.register_setting('-d', '--dest', dest='destdir', help='Specify the destination directory. [default: <hypervisor>-<distro>].')
+        self.register_setting('-c', '--config',  type='string', help='Specify a additional configuration file')
         self.register_setting('--debug', action='callback', callback=log.set_verbosity, help='Show debug information')
         self.register_setting('-v', '--verbose', action='callback', callback=log.set_verbosity, help='Show progress information')
         self.register_setting('-q', '--quiet', action='callback', callback=log.set_verbosity, help='Silent operation')
         self.register_setting('-t', '--tmp', default=os.environ.get('TMPDIR', '/tmp'), help='Use TMP as temporary working space for image generation. Defaults to $TMPDIR if it is defined or /tmp otherwise. [default: %default]')
+        self.register_setting('--templates', metavar='DIR', help='Prepend DIR to template search path.')
         self.register_setting('-o', '--overwrite', action='store_true', default=False, help='Force overwrite of destination directory if it already exist. [default: %default]')
-        self.register_setting('--in-place', action='store_true', default=False, help='Install directly into the filesystem images. This is needed if your \$TMPDIR is nodev and/or nosuid, but will result in slightly larger file system images.')
+        self.register_setting('--in-place', action='store_true', default=False, help='Install directly into the filesystem images. This is needed if your $TMPDIR is nodev and/or nosuid, but will result in slightly larger file system images.')
         self.register_setting('--tmpfs', metavar="OPTS", help='Use a tmpfs as the working directory, specifying its size or "-" to use tmpfs default (suid,dev,size=1G).')
         self.register_setting('-m', '--mem', type='int', default=128, help='Assign MEM megabytes of memory to the guest vm. [default: %default]')
+        self.register_setting('--cpus', type='int', default=1, help='Number of virtual CPU's. [default: %default]')
 
         group = self.setting_group('Network related options')
         domainname = '.'.join(socket.gethostbyname_ex(socket.gethostname())[0].split('.')[1:])
-        group.add_option('--domain', default=domainname, help='Set DOMAIN as the domain name of the guest. Default: The domain of the machine running this script: %default.')
-        group.add_option('--ip', metavar='ADDRESS', default='dhcp', help='IP address in dotted form [default: %default]')
+        group.add_option('--domain', metavar='DOMAIN', default=domainname, help='Set DOMAIN as the domain name of the guest [default: The domain of the machine running this script: %default].')
+        group.add_option('--ip', metavar='ADDRESS', default='dhcp', help='IP address in dotted form [default: %default].')
+        group.add_option('--mac', metavar='VALUE', help='MAC address of the guest [default: one will be automatically generated on first run].')
         group.add_option('--mask', metavar='VALUE', help='IP mask in dotted form [default: based on ip setting]. Ignored if --ip is not specified.')
         group.add_option('--net', metavar='ADDRESS', help='IP net address in dotted form [default: based on ip setting]. Ignored if --ip is not specified.')
         group.add_option('--bcast', metavar='VALUE', help='IP broadcast in dotted form [default: based on ip setting]. Ignored if --ip is not specified.')
@@ -235,7 +248,13 @@ class VM(object):
             for (k,v) in settings.__dict__.iteritems():
                 confvalue = self.get_conf_value(k)
                 if confvalue:
-                    setattr(self, k, confvalue)
+                    if self.optparser.get_option('--%s' % k):
+                        if self.optparser.get_option('--%s' % k).action == 'append':
+                            setattr(self, k, confvalue.split(', '))
+                        else:
+                            setattr(self, k, confvalue)
+                    else:
+                        setattr(self, k, confvalue)
                 else:
                     setattr(self, k, v)
 
@@ -249,8 +268,18 @@ class VM(object):
         """
 
         logging.debug("ip: %s" % self.ip)
+        
+        if self.mac:
+            valid_mac_address = re.compile("([0-9a-f]{2}:){5}([0-9a-f]{2})", re.IGNORECASE)
+            if not valid_mac_address.search(self.mac):
+                raise VMBuilderUserError("Malformed MAC address entered: %s" % self.mac)
+            else:
+                logging.debug("Valid mac given: %s" % self.mac)
 
         if self.ip != 'dhcp':
+            if self.domain == '':
+                raise VMBuilderUserError('Domain is undefined and host has no domain set.')
+
             try:
                 numip = struct.unpack('I', socket.inet_aton(self.ip))[0] 
             except socket.error:
@@ -288,7 +317,6 @@ class VM(object):
             logging.debug("gateway: %s" % self.gw)
             logging.debug("dns: %s" % self.dns)
 
-
     def create_directory_structure(self):
         """Creates the directory structure where we'll be doing all the work
 
@@ -342,6 +370,9 @@ class VM(object):
         fss = disk.get_ordered_filesystems(self)
         for fs in fss:
             fs.mount()
+            self.distro.post_mount(fs)
+
+        self.fsmounted = True
 
     def umount_partitions(self):
         """Unmounts all the vm's partitions and filesystems"""
@@ -353,6 +384,8 @@ class VM(object):
         for disk in self.disks:
             disk.unmap()
 
+        self.fsmounted = False
+
     def install(self):
         if self.in_place:
             self.installdir = self.rootmnt
@@ -361,6 +394,8 @@ class VM(object):
 
         logging.info("Installing guest operating system. This might take some time...")
         self.distro.install(self.installdir)
+
+        self.call_hooks('post_install')
     
         if not self.in_place:
             logging.info("Copying to disk images")
@@ -371,9 +406,25 @@ class VM(object):
             self.distro.install_bootloader()
 
     def preflight_check(self):
+        for opt in sum([self.confparser.options(section) for section in self.confparser.sections()], []) + [k for (k,v) in self.confparser.defaults().iteritems()]:
+            if '-' in opt:
+                raise VMBuilderUserError('You specified a "%s" config option in a config file, but that is not valid. Perhaps you meant "%s"?' % (opt, opt.replace('-', '_')))
+
         self.ip_defaults()
         self.call_hooks('preflight_check')
 
+    def install_file(self, path, contents=None, source=None, mode=None):
+        fullpath = '%s%s' % (self.installdir, path)
+        if source and not contents:
+            shutil.copy(source, fullpath) 
+        else:
+            fp = open(fullpath, 'w')
+            fp.write(contents)
+            fp.close()
+        if mode:
+            os.chmod(fullpath, mode)
+        return fullpath
+
     def create(self):
         """
         The core vm creation method
@@ -394,10 +445,9 @@ class VM(object):
         """
         util.checkroot()
 
-        self.preflight_check()
-    
         finished = False
         try:
+            self.preflight_check()
             self.create_directory_structure()
 
             disk.create_partitions(self)
@@ -419,7 +469,7 @@ class VM(object):
             raise e
         finally:
             if not finished:
-                logging.critical("Oh, dear, an exception occurred")
+                logging.debug("Oh, dear, an exception occurred")
             self.cleanup()
 
 class _MyOptParser(optparse.OptionParser):
@@ -457,6 +507,8 @@ class _MyOptParser(optparse.OptionParser):
             formatter.indent()
             result.append(self.format_arg_help(formatter))
             result.append("\n")
+            result.append("*** Use vmbuilder <hypervisor> <distro> --help to get more options. Hypervisor, distro, and plugins specific help is only available when the first two arguments are supplied.\n")
+            result.append("\n")
             formatter.dedent()
         result.append(formatter.format_heading(_("Options")))
         formatter.indent()
